<?php

namespace Teknisa\DF\Modules\Entrada\Services;

class ChaveAcesso {

    protected $entityManager;

    private $intgNotafiscal;
    private $filial;
    private $lancamento;
    private $nracessonfe;
    private $importacaoIgualXml;
    private $quantidadeProdutoXml = array();
    private $relacionamentoPorFilial = false;
    private $lancamentoEntradaFactory;

    public function __construct($entityManager, $filial, $lancamento, $nracessonfe, $lancamentoService, $rowLancamento) {
        $this->entityManager = $entityManager;
        $this->setNotaFiscalEntity($filial, $lancamento, $nracessonfe);
        try {
             $this->relacionamentoPorFilial = $this->utilizaRelacionamentoPorFilial($this->intgNotafiscal->getCdforneced());
        }
        catch(\Exception $e) {
             $this->relacionamentoPorFilial = false;
        }
        $this->lancamentoService = $lancamentoService;
        $this->rowLancamento     = $rowLancamento;
        $this->lancamentoEntradaFactory = new \Teknisa\DF\Modules\Entrada\Factories\Dfe11000NfEntradaFactory($this->entityManager);
    }

    public static function parseNullToZero($value) {
        return !!$value ? $value : 0;
    }

    public function setNotaFiscalEntity($filial, $lancamento, $nracessonfe) {
	    $this->filial = $filial;
        $this->lancamento = $lancamento;
        $this->nracessonfe = $nracessonfe;
        $this->quantidadeProdutoXml = array();

        $this->intgNotafiscal = $this->entityManager->getRepository('\Fiscal\Entities\IntgNotafiscal')
            ->findOneBy( array('nracessonfe' => $this->nracessonfe) );
	}

	private function retornaNotaFiscalImportada($filial, $lancamento) {
	    $notaFiscal = $this->entityManager->getRepository('\Fiscal\Entities\Notafiscal')
            ->findOneBy( array('cdfilial' => $filial, 'nrlanctonf' => $lancamento) );
        return $notaFiscal;
	}

	private function criaValorNrnotafiscalele($nrnotafisc) {
	    $nrnotafiscalele = str_pad($nrnotafisc, 20, "0", STR_PAD_LEFT);
        $nrnotafiscalele = substr($nrnotafiscalele, 11, 19);

	    return $nrnotafiscalele;
	}

	private function importacaoIntgNotaFiscal() {
	    $filial = $this->filial;
        $lancamento = $this->lancamento;
        
	    $notaFiscal      = $this->retornaNotaFiscalImportada($filial, $lancamento);
        $nrnotafiscalele = $this->criaValorNrnotafiscalele($notaFiscal->getNrnotafisc());

	    $this->lancamentoEntradaFactory->updateNrnotafiscalele($notaFiscal, $nrnotafiscalele);

        return $this->intgNotafiscal;
	}

	private function somaQuantidadeProdutos($intgItemproduto) {
	    $this->quantidadeProdutoXml = array();
        foreach ($intgItemproduto as $item) {
            $cdproduto = $item->getCdproduto();
            $novaQuantidade = self::parseNullToZero($item->getQtitemprod());
            $quantidade = 0;

            if (array_key_exists($cdproduto, $this->quantidadeProdutoXml) && !$this->importacaoIgualXml) {
                $quantidade = $this->quantidadeProdutoXml[$cdproduto];
            }

            $key = $cdproduto . ($this->importacaoIgualXml? '_'.$item->getNrseqitprod() : '');
            $this->quantidadeProdutoXml[$key] = $quantidade + $novaQuantidade;
        }
	}

	private function importacaoIntgItemProduto() {
        $intgItemproduto = $this->entityManager->getRepository('\Fiscal\Entities\IntgItemproduto')
            ->findBy( array('nracessonfe' => $this->nracessonfe) );

        $filial = $this->filial;
        $lancamento = $this->lancamento;

        $this->somaQuantidadeProdutos($intgItemproduto);

        $notaFiscal = $this->retornaNotaFiscalImportada($filial, $lancamento);

        $lancamentoNotaFiscalFactory = new \Teknisa\DF\Factories\NotafiscalLancamentoFactory($this->entityManager);

        foreach ($intgItemproduto as $item) {
            $nrseqitprod = str_pad($item->getNrseqitprod(), 4, '0', STR_PAD_LEFT);
            $cdprodutoXml = $item->getCdproduto();
            $productEntity = $this->getProdutoCorrespondenteXml($cdprodutoXml, $notaFiscal->getCdforneced(), $notaFiscal->getNracessonfe(), $notaFiscal->getCdfilirece());

            if (is_null($productEntity)) {
                $cdproduto = $cdprodutoXml;
                $cdprodemis = $cdproduto;
                $nmproduto = $item->getNmproduto();
            } else {
                $cdproduto = $productEntity->getCdproduto();
                $cdprodemis = $productEntity->getCdarvprod();
                $nmproduto = $productEntity->getNmproduto();
            }
            
            $params = array(
                'cdfilial' => $filial,
                'nrlanctonf' => $lancamento,
                'cdproduto' => $cdproduto
            );
            
            $itemCadastrado = $this->entityManager->getRepository('\Geral\Entities\Itemproduto')
                ->findOneBy($params);
            
            $key = $cdprodutoXml . ($this->importacaoIgualXml? '_'.$item->getNrseqitprod() : '');
            $qtitemprod = self::parseNullToZero($item->getQtitemprod());
            $vrunititem = self::parseNullToZero($item->getVrunititem());
            $vrdescitem = self::parseNullToZero($item->getVrdescitem());
            $valorTotal = $this->importacaoIgualXml? self::parseNullToZero($item->getVrtotaitem()) : $qtitemprod * $vrunititem;
            $vrtotaitem = $valorTotal; //self::parseNullToZero($item->getVrtotaitem());
            $vrbrutitem = self::parseNullToZero($item->getVrbrutitem());
            $vrcontitem = self::parseNullToZero($item->getVrcontitem());

            $cdmarca = $item->getCdmarca();
            $dsprodemis = $nmproduto;
            $sgunidade = $item->getSgunidade();
            $nracessonfe = $item->getNracessonfe();
            $cdfisopeitem = $item->getCdfisopeitem();
            $cdsitutribpr = $item->getCdsitutribpr();
            $organizacao = $item->getNrorg();
            $operador = $item->getCdoperinclusao();
            $cdfilimovi = $notaFiscal->getCdfilirece();
            $nrautoforn = null;
            
            //TODO não instanciar a classe
            $lancamentoNotaFiscalFactory->novoItemproduto($filial, $lancamento, $nrseqitprod, $cdproduto, $cdprodemis,
                $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
                $vrtotaitem, $organizacao, $operador, $nrautoforn, $cdfilimovi, $cdmarca, null, $cdfisopeitem
            );
            
            $this->entityManager->flush();
        }
	}

	private function importacaoIntgtrabitproimpos() {
	    
            // Iniciamos o "contador"
        list($usec, $sec) = explode(' ', microtime());
        $script_start = (float) $sec + (float) $usec;
	    
	    $intgTrabitproimpos = $this->getIntgTrabitProImpos($this->nracessonfe);

        $filial = $this->filial;
        $lancamento = $this->lancamento;
        $produtos = array();

        $entradaFactory = new \Teknisa\DF\Modules\Entrada\Factories\Dfe11000NfEntradaFactory($this->entityManager);

        foreach($intgTrabitproimpos as $imposto) {
            $seqImposto   = str_pad($imposto["NRSEQIMPOIT"], 4, '0', STR_PAD_LEFT);
            $cdimpostoit  = $imposto["CDIMPOSTOIT"];
            
            $params = array(
                'cdimposto' => $cdimpostoit
            );
            $taxEntity = $this->entityManager->getRepository('\Geral\Entities\Imposto')
                ->findOneBy($params);
                
            $valorinfo = ($this->importacaoIgualXml || $taxEntity->getIdimportaimpxml() == 'S') ? 'S' : 'N';
                
            if($valorinfo == 'S') {
                $vrbasecalcit = self::parseNullToZero($imposto["VRBASECALCIT"]);
                $vraliqitem   = self::parseNullToZero($imposto["VRALIQITEM"]);
                $vrimpostoit  = self::parseNullToZero($imposto["VRIMPOSTOIT"]);
                $cdfiscalit   = $imposto["CDFISCALIT"];
                $organizacao  = $imposto["NRORG"];
                $operador     = $imposto["CDOPERINCLUSAO"];
                $vrAdicFecp   = $imposto["VRADICFECP"];
                $vrAdicAliqFecp = $imposto["VRADICALIQFECP"];
                $cdstribimpoit = $imposto["CDSITUTRIBIT"];
                //CDSTRIBIMPOIT Gravar
    
                $params = array(
                    'cdfilimpoit' => $filial,
                    'nrlancimpoit' => $lancamento,
                    'nrseqimpoit' => $seqImposto,
                    'cdimpostoit' => $cdimpostoit
                );
                $impostoCadastrado = $this->entityManager->getRepository('\Fiscal\Entities\Trabitproimpos')
                    ->findOneBy( $params );
                
                if ($impostoCadastrado) {
                    $alteracaoUser = $impostoCadastrado->getIdavisimpoit();
                    $tempTaxEntity = $entradaFactory->editarImpostoItemTemp(
                        $filial, $lancamento, $seqImposto, $cdimpostoit, $vrimpostoit, $vraliqitem, $alteracaoUser,
                        $organizacao, $operador, $vrbasecalcit, 'S', $vrAdicFecp, $vrAdicAliqFecp, $valorinfo, $cdstribimpoit
                    );
                } else {
                    $tempTaxEntity = $entradaFactory->novoImpostoItemTemp(
                        $filial, $lancamento, $seqImposto, $cdimpostoit, $organizacao, $operador, $vrimpostoit,
                        $vraliqitem, $vrbasecalcit, 'N',  1,  0,  0,  0, 'N',
                        '9',  'N', $vrAdicFecp, $vrAdicAliqFecp, $valorinfo, $cdstribimpoit
                    );
                }
                if(!isset($produtos[$seqImposto])) {
                    $produtos[$seqImposto] = array(
                        "GERAIMPOSTO" => false,
                        "IMPORTADOS" => array(
                            $cdimpostoit
                        )
                    );
                }
                else {
                    array_push($produtos[$seqImposto]["IMPORTADOS"], $cdimpostoit);
                }
            }
            else {
                
               if(!isset($produtos[$seqImposto])) {
                    $produtos[$seqImposto] = array(
                        "GERAIMPOSTO" => true,
                        "IMPORTADOS" => array()
                    );
                }
                else {
                    $produtos[$seqImposto]["GERAIMPOSTO"] = true;
                }
            }

        }
        
        foreach($produtos as $nrSeqItem => $impostos) {
            if($produtos[$nrSeqItem]["GERAIMPOSTO"] === true) {
               $params = array(
                    'P_CDFILIAL' => $filial,
                    'P_NRLANCTONF' => $lancamento,
                    'P_NRSEQITPROD' => $nrSeqItem
                );
                $item = $entradaFactory->getItemProduto($params);
                $impostos   = $this->lancamentoService->calculaImpostoItemProduto($this->rowLancamento, $item, $impostos["IMPORTADOS"]);   
            }
                
        }
    
	}

	private function importacaoIntgItemEncargo() {
        $intgItemencargo = $this->entityManager->getRepository('\Fiscal\Entities\IntgItemencargo')
            ->findBy( array('nracessonfe' => $this->nracessonfe) );

        $lancamentoFactory = new \Teknisa\DF\Factories\NotafiscalLancamentoFactory($this->entityManager);

        $filial = $this->filial;
        $lancamento = $this->lancamento;

        foreach ($intgItemencargo as $encargo) {
            $cdencargo   = $encargo->getCdencargo();
            $vritemenca  = $encargo->getVritemenca();
            $organizacao = $encargo->getNrorg();
            $operador    = $encargo->getCdoperinclusao();
            $params = array(
                'cdfilial' => $filial,
                'nrlanctonf' => $lancamento,
                'cdencargo' => $cdencargo
            );
            $encargoCadastrado = $this->entityManager->getRepository('\Geral\Entities\Itemencargo')
                ->findOneBy( $params );

            if ($encargoCadastrado) {
                return $lancamentoFactory->editarEncargo(
                    $encargoCadastrado, $vritemenca, $organizacao, $operador
                );
            }

            $lancamentoFactory->novoEncargo(
                $filial, $lancamento, $cdencargo, $vritemenca, $organizacao, $operador
            );
        }
	}

	private function importacaoIntgContaPagar() {
	    
	    if($this->fornecedorImportaVencimento($this->rowLancamento["CDFORNECED"])) {
            $intgContapagar = $this->entityManager->getRepository('\Fiscal\Entities\IntgContapagar')
                ->findBy( array('nracessonfe' => $this->nracessonfe) );
    
            $filial = $this->filial;
            $lancamento = $this->lancamento;
    
            $entradaFactory = new \Teknisa\DF\Modules\Entrada\Factories\Dfe11000NfEntradaFactory($this->entityManager);
    
            foreach ($intgContapagar as $conta) {
                $nrseqvenpag    = $conta->getNrseqvenpag();
                $nrparcelapag   = $conta->getNrparcelapag();
                $cdtpctpagar    = $conta->getCdtpctpagar();
                $qtdiasvenpag   = $conta->getQtdiasvenpag();
                $dtorigvenpag   = $conta->getDtorigvenpag();
                $dtatuavenpag   = $dtorigvenpag;
                $vrorigpag      = $conta->getVrorigpag();
                $vratuapag      = $vrorigpag;
                $cdempresa      = $conta->getCdempresa();
                $organizacao    = $conta->getNrorg();
                $operador       = $conta->getCdoperinclusao();
    
                $entradaFactory->novoContaPagar(
                    $filial, $lancamento, $nrseqvenpag, $nrparcelapag, $cdtpctpagar,
                    $qtdiasvenpag, $dtorigvenpag, $dtatuavenpag, $vrorigpag,
                    $vratuapag, $cdempresa, $organizacao, $operador
                );
            }
	    }

	}

	private function verificaImportacaoIdenticaXml() {
	    $paramgeral = $this->entityManager->getRepository('\Geral\Entities\Paramgeral')
            ->findOneBy(array());

        $this->importacaoIgualXml = ($paramgeral->getIdgernfigualxml() === 'S');
	}

	private function getProdutoCorrespondenteXml($cdprodutoXml, $fornecedor, $nracessonfe, $filial) {
	    $cdProduto = null;
        
        $produto = $this->entityManager->getRepository('\Geral\Entities\Produto')
                ->findOneBy(array('cdarvprod' => $cdprodutoXml));
        
        if($this->relacionamentoPorFilial) {
            $prodfornxml = $this->getProdFornXmlFil($cdprodutoXml, $fornecedor, $nracessonfe, $filial);
        }
        else {
            $prodfornxml = $this->getProdForn($cdprodutoXml, $fornecedor);
        }
        
        if(isset($prodfornxml[0])) {
            $cdProduto = $prodfornxml[0]["CDPRODUTO"];
        } else if(isset($produto)){
            $cdProduto = $produto->getCdproduto();
        }
        
        if ($cdProduto !== null) {
            return $this->entityManager->getRepository('\Geral\Entities\Produto')
                ->findOneBy(array('cdproduto' => $cdProduto));
        } else return null;
	}
	
	public function utilizaRelacionamentoPorFilial($fornecedor) {
        $sql = "SELECT PI.IDUTRELXMLFIL,
                  PF.IDBLOQALTPRODREL
                FROM PARAMINTEGRA PI,
                  PARAMFORNE PF
                WHERE PF.CDFORNECED = :P_CDFORNECED";
                
        $params = array("P_CDFORNECED" => $fornecedor);

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
        $paramIntegra = $std->fetchAll();
        
        if(isset($paramIntegra[0])) {
            $paramIntegra = $paramIntegra[0];
            if($paramIntegra["IDUTRELXMLFIL"]) {
                return ($paramIntegra["IDUTRELXMLFIL"] == 'S' && $paramIntegra["IDBLOQALTPRODREL"] == 'N');   
            }
            else return false;
        }
        else return false;
    }
    
    public function fornecedorImportaVencimento($fornecedor) {
        $sql = "SELECT IMPVENCXML FROM PARAMFORNE WHERE CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDFORNECED"  => $fornecedor 
        ); 
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        $fornecedor = $std->fetch();
        return $fornecedor["IMPVENCXML"] == 'S';
    }
	
	public function getProdForn($cdprodutoXml, $fornecedor) {
        $sql = "SELECT CDPRODUTO FROM PRODFORNXML WHERE CDFORNECED = :P_CDFORNECED AND CDPRODUTOXML = :P_CDPRODUTOXML";
        
        $params = array("P_CDFORNECED"  => $fornecedor,
                        "P_CDPRODUTOXML"=> $cdprodutoXml);

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function getIntgTrabitProImpos($nracessonfe) {
        $sql = "SELECT * FROM INTG_TRABITPROIMPOS WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array("P_NRACESSONFE"  => $nracessonfe);

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function getFirstItem($filial, $lancamento) {
        return $this->lancamentoEntradaFactory->getFirstItem($filial, $lancamento);
    }
    
    public function getProdFornXmlFil($cdprodutoXml, $fornecedor, $nracessonfe, $filial) {
        $sql = "SELECT  NVL(PF.CDPRODUTOXML, P.CDPRODUTOXML) AS CDPRODUTOXML,
                        NVL(PF.CDPRODUTO, P.CDPRODUTO)       AS CDPRODUTO
                FROM INTG_ITEMPRODUTO I,
                  PRODFORNXML P,
                  PRODFORNXMLFIL PF,
                  PRODUTO PR
                WHERE I.NRACESSONFE    = :P_NRACESSONFE
                AND P.CDFORNECED(+)    = :P_CDFORNECED
                AND P.CDPRODUTOXML(+)  = I.CDPRODUTO
                AND PF.CDFORNECED(+)   = :P_CDFORNECED
                AND PF.CDFILIAL(+)     = :P_CDFILIAL
                AND PF.CDPRODUTOXML(+) = I.CDPRODUTO
                AND P.CDPRODUTOXML(+)  = I.CDPRODUTO
                AND I.CDPRODUTO        = :P_CDPRODUTOXML
                AND PR.CDPRODUTO (+)   = P.CDPRODUTO
                ORDER BY TO_NUMBER(I.NRSEQITPROD)";
                
        $params = array("P_CDFORNECED"  => $fornecedor,
                        "P_CDPRODUTOXML"=> $cdprodutoXml,
                        "P_NRACESSONFE" => $nracessonfe,
                        "P_CDFILIAL"    => $filial);

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
	

	public function realizaImportacao() {
	    try {
	        $this->verificaImportacaoIdenticaXml();
            $this->importacaoIntgNotaFiscal();
            $this->importacaoIntgItemProduto();
            $this->importacaoIntgtrabitproimpos();
            $this->importacaoIntgItemEncargo();
            $this->importacaoIntgContaPagar();
	    } catch (\Exception $e) {
	        throw $e;
	    }
	}
}
