<?php
namespace Teknisa\DF\Modules\Saida\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use NFePHP\NFe\Tools as NFe;
use NFePHP\Common\Certificate;

use Teknisa\DF\Modules\Saida\Factories\Dfs51000InutilizacaoNotaFiscalEletronicaFactory;
use Teknisa\DF\Services\NFe\ReturnNFe;

class Dfs51000InutilizacaoNotaFiscalEletronicaService{
    protected $environment;
    protected $dataSourceManager;
    protected $nfeTools;
    protected $aResposta;
    protected $dfs51000InutilizacaoNotaFiscalEletronicaFactory;

    public function __construct(Dfs51000InutilizacaoNotaFiscalEletronicaFactory $dfs51000InutilizacaoNotaFiscalEletronicaFactory,
                                $crudService, Environment $environment, \Doctrine\ORM\EntityManager $entityManager, $dataSourceManager,
                                $fileServerLink){
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->dfs51000InutilizacaoNotaFiscalEletronicaFactory = $dfs51000InutilizacaoNotaFiscalEletronicaFactory;
        $this->fileServerLink = $fileServerLink;
    }

    public function buscarAgendamento($conditions){
        try {
            $conditions->addCondition('NRORG',$this->environment->getNrOrgTrab());
            $conditions->setDataSourceName('gravaintinutnfe_inutilizacao');
            $parameterBag = array("P_CDEMPRESA","P_CDSERNFE","P_NRCNPJINSESTNFE","P_IDTPAMBIENTENFE");
            
            $dataSet = $this->realizaConsultaBancoFilterCriteria($conditions,$parameterBag);
            
            $ultimaConsulta = '';
            $proximaConsulta = '';
            $situacaoAgendamento = '';
            if(!empty($dataSet->getRows())){
                $ultimaConsulta = $dataSet->getRows()[0]['DTHRCONSINUTNFE'];
            }
            $consultaData= $this->realizaConsultaBanco([],'all_jobs_inutulizacaonf',array());
            if( empty($consultaData->getRows() ) ){
                $situacaoAgendamento = 'I';

            }else{
                $situacaoAgendamento = 'A';
                $proximaConsulta = $ $consultaData->getRows()[0]['DTHRPROXIMACONSULTA '];
                if(!$ultimaConsulta){
                    $ultimaConsulta = $ $consultaData->getRows()[0]['DTHRULTIMACONSULTA '];
                }
            }
            $array = array(
                "ultimaConsulta" => $ultimaConsulta,
                "proximaConsulta" => $proximaConsulta,
                "situacaoAgendamento"=>$situacaoAgendamento
            );
            $data = new DataSet('data', $array);
            return $data;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function confirmarInutilizacao($config, $rows){
        try {
            $config = $this->recolocarArrobaEmail($config);
            $this->nfeTools = $this->getToolsNFe($config);
            $ambiente = $this->nfeTools->config->tpAmb;
            
            foreach ($rows as &$row) {
                $this->aResposta = array();
                $row['NRINICIAL'] = ltrim($row['NRINICIAL'], '0');
                $row['NRFINAL']   = ltrim($row['NRFINAL']  , '0');
                $resp = $this->nfeTools->sefazInutiliza($row['CDSERNFE'], $row['NRINICIAL'], $row['NRFINAL'], $row['JUSTIFICATIVA'], $ambiente, $this->aResposta, true);
                $this->aResposta = ReturnNFe::readReturnSefaz('NfeInutilizacao', $resp);
                
                //resposta caso seja homologada a inutilizacao
                if($this->aResposta['cStat'] == '102') {

                    // É NECESSARIO GRAVAR O NUMERO COM OS ZEROS A ESQUERDA !!!!
                    $tamanho = strlen($row['INTERVALOINI']);
                    $nrini = intval($row['INTERVALOINI']);
                    $nrfin = intval($row['INTERVALOFIN']);

                    for($numero = $nrini; $numero <= $nrfin; $numero++){
                        $num = str_pad((string)$numero, $tamanho, "0", STR_PAD_LEFT);
                        $this->insNumInut($row['CDEMPRESA'], $row['NRCNPJINSESTNFE'],$row['CDSERNFE'], $num, $ambiente, DateUtil::getDataDeString(substr($this->aResposta['dhRecbto'], 0, 10)), $row['CDINSCESTA'], $row['CDINSCMUNI'], $this->aResposta['nProt'], '1', $row['JUSTIFICATIVA']);
                    }
                    $this->updGravaIntInutNFe('A', $this->aResposta['xMotivo'], $row['CDEMPRESA'], $row['CDSERNFE'], $row['INTERVALOINI'], $row['INTERVALOFIN'], $row['NRCNPJINSESTNFE'], $ambiente);

                }
                //resposta caso seja negada a inutilizacao
                else{
                    $this->updGravaIntInutNFe('R', $this->aResposta['xMotivo'], $row['CDEMPRESA'], $row['CDSERNFE'], $row['INTERVALOINI'], $row['INTERVALOFIN'], $row['NRCNPJINSESTNFE'], $ambiente);
                }
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function insNumInut ($empresa, $cnpj, $serie, $numero, $ambiente, $data, $inscricaoEstadual, $inscricaoMunicipal, $prot, $tipoEmissao, $justificativa){
        try {
    		$row = new Row();
    		$nrorg = $this->environment->getNrorgTrab();
        	$operador = $this->environment->getCdOperador();
        	$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $row->set('__is_new',true);
            $row->set('NRORG',$nrorg);
            $row->set('NRORGULTATU',$nrorg);
            $row->set('NRORGINCLUSAO',$nrorg);
            $row->set('DTULTATU',$currentTime);
            $row->set('DTINCLUSAO',$currentTime);
            $row->set('CDOPERINCLUSAO',$operador);
            $row->set('CDOPERULTATU',$operador);
            $row->set('IDATIVO','S');
            $row->set('CDEMPRESA',$empresa);
            $row->set('NRINSJURIESTNFE',$cnpj);
            $row->set('CDSERIE',$serie);
            $row->set('NRINUTILNFELE',$numero);
            $row->set('IDTPAMBNFENUM',$ambiente);
            $row->set('DTINUTNFE',$data);
            $row->set('CDINSCESTA',$inscricaoEstadual);
            $row->set('CDINSCMUNI',$inscricaoMunicipal);
            $row->set('NRPROTOCINUTNFE',$prot);
            $row->set('IDTPEMISINUTIL',$tipoEmissao);
            $row->set('DSJUSNUMINUTILNFE',$justificativa);
            $array[] = $row;

            $dataSet = new DataSet('numinutilnfele', $array);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();

        }catch (\Exception $e) {
           	throw $e;
        }
    }

    public function updGravaIntInutNFe($status, $observacao, $empresa, $serie, $intervaloIni, $intervaloFin, $cnpj, $ambiente){
        try {
    		$sql = <<<SQL
            UPDATE GRAVAINTINUTNFE
               SET IDSTATUS = :P_IDSTATUS,
                   DSOBSERVACAO = :P_DSOBSERVACAO
             WHERE CDEMPRESA = :P_CDEMPRESA
               AND CDSERNFE = :P_CDSERNFE
               AND INTERVALOINI = :P_INTERVALOINI
               AND INTERVALOFIN = :P_INTERVALOFIN
               AND NRCNPJINSESTNFE = :P_NRCNPJINSESTNFE
               AND IDTPAMBIENTENFE = :P_IDTPAMBIENTENFE
               AND NRORG = :P_NRORG
SQL;
            $params = array(
                'P_IDSTATUS'      => $status,
                'P_DSOBSERVACAO'   => $observacao,
                'P_CDEMPRESA'      => $empresa,
                'P_CDSERNFE'      => $serie,
                'P_INTERVALOINI'      => $intervaloIni,
                'P_INTERVALOFIN'      => $intervaloFin,
                'P_NRCNPJINSESTNFE'      => $cnpj,
                'P_IDTPAMBIENTENFE'      => $ambiente,
                'P_NRORG'      =>  $this->environment->getNrorgTrab()
            );

            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);

        }catch (\Exception $e) {
           	throw $e;
        }


    }

    public function gerarNovaConsulta($config, $row){
        try {
            $this->nfeTools = $this->getToolsNFe($config);

            $ambiente = $this->nfeTools->config->tpAmb;
            $empresa            = $row['CDEMPRESA'];
            $cnpj               = $row['NRINSJURIEST'];
            $inscricaoEstadual  = $row['CDINSCESTA'];
            $inscricaoMunicipal = $row['CDINSCMUNI'];
            $serieNfe           = $row['CDSERIENFE'];
            
            $this->callProcedureInutilizacao($empresa, $cnpj, $inscricaoEstadual, $inscricaoMunicipal, $serieNfe, $ambiente);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getToolsNFe($config) {
        $certificado = $this->dfs51000InutilizacaoNotaFiscalEletronicaFactory->getCertificado($config);
    	$certificadoEncontrado = false;
    	
    	foreach($certificado as $cert) {
    	    if($cert["DSDIRCERTIFICADO"]) {
    	        $certificadoEncontrado = true;
    	        $certify = $cert;
    	    }
    	}
	    if($certificadoEncontrado) {
            $senha = $this->decrypt($certify["CDSENHACERTIF"]);
    		$certificado = $certify["DSDIRCERTIFICADO"];
    		$certificado = explode('>', $certificado);
            $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
	    } else {
	       throw new \Exception("Certificado Digital não encontrado.");
	    }
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
            $this->certificate = $certificado;
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
        $tools = new NFe(json_encode($config), $certificado);
        $tools->soap->httpVersion('1.1');
        
        return $tools;
	}
	
	public function decrypt($text, $salt = 'teknisa') {
          return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function findInutilizacao($filtro){
        try {
            $resul = $this->findFilial($filtro);
            $dataSet = $this->findInutilizacaoNF($resul->getRows()[0],$filtro->getConditions());
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function findFilial ($filtro){
        try {
            $parameterBag = array("CDEMPRESA","CDINSCESTA","CDFILIAL");
            $conditions = $this->condicaoFiltroFilial($filtro->getConditions());
            $dataSet = $this->realizaConsultaBanco($conditions,'filial_endefili_estado',$parameterBag);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function findInutilizacaoNF($resul, $filtro){
        try {
            $parameterBag = array("P_CDEMPRESA","P_CDSERNFE","P_NRCNPJINSESTNFE","P_IDTPAMBIENTENFE");
            $conditions = $this->condicaoInutilizacaoNF($resul, $filtro);
            $dataSet = $this->realizaConsultaBanco($conditions,'gravaintinutnfe_inutilizacao',$parameterBag);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function condicaoInutilizacaoNF($resul, $filtro){
        $empresa = array();
        $serie = array();

        for($i=0; $i<count($filtro); $i++){
            if( $filtro[$i]['columnName'] == 'CDEMPRESA') {
                array_push($empresa, $filtro[$i]);
            } else if( $filtro[$i]['columnName'] == 'CDSERIE') {
                    array_push($serie, $filtro[$i]);
            }
        }

        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $resul['NRINSJURIEST'],
            "operator" => "=",
            "columnName" => "P_NRCNPJINSESTNFE"
          ),
          array(
            "value" => $resul['IDTPAMBIENTENFE'],
            "operator" => "=",
            "columnName" => "P_IDTPAMBIENTENFE"
          ),
          array(
            "value" => $empresa[0]['value'],
            "operator" => "=",
            "columnName" => "P_CDEMPRESA"
          ),
          array(
            "value" => $serie[0]['value'],
            "operator" => "=",
            "columnName" => "P_CDSERNFE"
          )
        );
        return $conditions;
    }

    public function condicaoFiltroFilial($filtro) {
        $conditions = array();
        $nrorgfiltro = array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
        );
        array_push($conditions, $nrorgfiltro);
        for($i=0; $i<count($filtro); $i++){
            if( $filtro[$i]['columnName'] == 'CDEMPRESA' || $filtro[$i]['columnName'] == 'CDINSCESTA' || $filtro[$i]['columnName'] == 'CDFILIAL' ) {
                array_push($conditions, $filtro[$i]);
            }
        }
        return $conditions;
    }

    public function realizaConsultaBanco($conditions,$dataSourceName,$parameterBag) {
        try {
            $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $parameterBag);
            $dataSet = $this->buscar($newFilterCriteria);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function realizaConsultaBancoFilterCriteria($filterCriteria,$parameterBag) {
        try {
            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $parameterBag);
            $dataSet = $this->buscar($newFilterCriteria);
            return $dataSet;
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscar($condicao){
        try {
            return $this->dataSourceManager->findBy($condicao);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    private function recolocarArrobaEmail($config){
       /* $config            = json_decode($config); //todo refatorar
        $config->aMailConf = json_decode(str_replace("&#64", "@", json_encode($config->MailConf)));
        $config            = json_encode($config);
        return $config;*/
        
        $config["aMailConf"] = str_replace("&#64", "@", $config["aMailConf"]);
        return $config;
    }

    public  function callProcedureInutilizacao($empresa, $cnpj, $inscricaoEstadual, $inscricaoMunicipal, $serieNfe, $ambiente, $modoOperacional = '1', $geral = 'N'){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'INUTILIZACAO_NFE');
            
            $procedure->addParam(new Param("P_CDEMPRESA",           Param::PARAM_INPUT,     $empresa,               Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_NRINSJURIEST",        Param::PARAM_INPUT,     $cnpj,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDINSCESTA",          Param::PARAM_INPUT,     $inscricaoEstadual,     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDINSCMUNI",          Param::PARAM_INPUT,     $inscricaoMunicipal,    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDSERIENFE",          Param::PARAM_INPUT,     $serieNfe,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_IDTPAMBIENTENFE",     Param::PARAM_INPUT,     $ambiente,              Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_MODOPERACIONAL",      Param::PARAM_INPUT,     $modoOperacional,       Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_GERAL",               Param::PARAM_INPUT,     $geral,                 Param::PARAM_TYPE_STR));

            $procedure->execute();

        } catch (\Exception $e) {
            throw $e;
        }
    }

}